/*    SpeedyType is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 3 of the License, or
      (at your option) any later version.

      SpeedyType is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with SpeedyType.  If not, see <http://www.gnu.org/licenses/>.

____________________________________________________________________________________

     Author: Federico Salotti
     Organization/Company: Mr_Robot_Computers_Repair
     Country: Italy Barga(LU)
     Release: 0.1 (27 September 2019 23:36)

*/



#include <SFML/Graphics.hpp>
#include <iostream>
#include <iomanip>
#include <cstring>
#include <algorithm>
#include <time.h>
#include <fstream>

#include "display.h"
#include "word.h"

#define WIDTH 896
#define HEIGHT 504

sf::Font font;

sf::Clock main_chrono;
sf::Clock game_chrono;
sf::Clock difficult_chrono;
sf::Clock spawn_chrono;
sf::Clock move_chrono;

sf::Text ui_lista_nome;

unsigned int tot_dizionario=0;
unsigned int  destroyed_w_counter=0;
unsigned int errors_counter=0;
float type_time=0.0;
float your_speed=0.0;
float move_time=0.5;
float spawn_time=3.0;

unsigned short lives=3;

bool finish=false;
bool is_game_start=false;
float game_time_final=0.0;

std::string u_input_string="";
std::vector<Word> words;


void populateVector(std::vector<Word> &words,sf::Font &font, char** argv)
{
	std::string line;
	std::ifstream reader;

	if(!argv[1]){ reader.open("word_list/default.txt"); ui_lista_nome.setString("DEFAULT");}
	else if(strcmp(argv[1], "nomi") == 0){ reader.open("word_list/nomi.txt"); ui_lista_nome.setString("NOMI");}
	else if(strcmp(argv[1], "composte") == 0){ reader.open("word_list/composte.txt"); ui_lista_nome.setString("COMPOSTE");}
	else if(strcmp(argv[1], "parole comuni") == 0){ reader.open("word_list/parole_comuni.txt"); ui_lista_nome.setString("PAROLE COMUNI");}
	else {reader.open("word_list/default.txt"); ui_lista_nome.setString("DEFAULT");}

	while(getline(reader,line))
	{
		Word w(font,line);
		 words.push_back(w);
	}

	std::random_shuffle(words.begin(), words.end());
	reader.close();
}


void drawWords(std::vector<Word> words, sf::RenderWindow &window)
{
	for(auto word=words.begin(); word != words.end(); ++word) if((*word).isDraw()) (*word).draw(window);
}

void moveWords(std::vector<Word> &words)
{
	for(auto word=words.begin(); word != words.end(); ++word) if((*word).isDraw())(*word).move();
}

void selectWords(std::vector<Word> &words)
{
	for(auto word=words.begin(); word != words.end(); ++word)
	{
		if( !(*word).isDraw() )
		{
			(*word).select();
			break;
		}
	}
}

void checkWordsPosition(std::vector<Word> &words)
{
	for(auto word=words.begin(); word != words.end(); ++word)
	{
		if((*word).getX() > WIDTH-200) (*word).warning();
		else if((*word).getX() > WIDTH-400) (*word).alert();

		if((*word).getX() >= WIDTH-100)
		{
			words.erase(word);
			--word;
			lives-=1;
		}
	}
}

void deleteWord(std::vector<Word> &words, std::string u_input)
{

	transform (u_input.begin(), u_input.end(), u_input.begin(), ::tolower);
	bool found=true;

	for(auto word=words.begin(); word != words.end(); ++word)
	{
		if((*word).getText().compare(u_input) == 0 && (*word).isDraw())
		{
			words.erase(word);
			--word;
			destroyed_w_counter+=1;
			type_time+=game_chrono.getElapsedTime().asSeconds();
			game_chrono.restart();
			break;
		}
		else found=false;
	}

	if(!found && u_input.size() > 0) errors_counter+=1;
}

bool isGameStart(std::vector<Word> &words)
{
	for(auto word : words)
	{
		if(word.isDraw()) return true;
	}
}

void restart(char** argv)
{

	is_game_start=false;
	lives=3;
	finish=false;
	move_time=0.5;
	spawn_time=3;
	your_speed=0;
	type_time=0.0;
	destroyed_w_counter=0;
	game_time_final=0.0;
	u_input_string.clear();

	words.clear();
	populateVector(words,font,argv);
	tot_dizionario=words.size();
	destroyed_w_counter=0;
	errors_counter=0;

	main_chrono.restart();
	game_chrono.restart();
	difficult_chrono.restart();
	spawn_chrono.restart();
	move_chrono.restart();
}

int main(int argc, char** argv)
{
	srand(time(nullptr));

	sf::RenderWindow window(sf::VideoMode(WIDTH,HEIGHT),"Speedy Type");
	window.setFramerateLimit(0);

	if(!font.loadFromFile("font/font.ttf")) return EXIT_FAILURE;

	sf::Text ui_dizionario_label;
	ui_dizionario_label.setFont(font);
	ui_dizionario_label.setString("DIZIONARIO : ");
	ui_dizionario_label.setCharacterSize(20);
	ui_dizionario_label.setFillColor(sf::Color::Blue);
	ui_dizionario_label.setPosition(10,50);

	sf::Text ui_destroy_w_label;
	ui_destroy_w_label.setFont(font);
	ui_destroy_w_label.setString("DISTRUTTE : ");
	ui_destroy_w_label.setCharacterSize(20);
	ui_destroy_w_label.setFillColor(sf::Color::Green);

	sf::Text game_time_label;
	game_time_label.setFont(font);
	game_time_label.setString("TIME : ");
	game_time_label.setCharacterSize(20);
	game_time_label.setFillColor(sf::Color::White);

	sf::Text errors_label;
	errors_label.setFont(font);
	errors_label.setString("ERRORI : ");
	errors_label.setCharacterSize(20);
	errors_label.setFillColor(sf::Color::Red);

	sf::Text ui_lista_nome_label;
	ui_lista_nome_label.setFont(font);
	ui_lista_nome_label.setString("LISTA : ");
	ui_lista_nome_label.setFillColor(sf::Color::Cyan);
	ui_lista_nome_label.setCharacterSize(20);
	ui_lista_nome_label.setPosition(WIDTH-280,50);

	sf::Text ui_game_time;
	ui_game_time.setFont(font);
	ui_game_time.setFillColor(sf::Color::White);
	ui_game_time.setCharacterSize(20);

	ui_lista_nome.setFont(font);
	ui_lista_nome.setFillColor(sf::Color::White);
	ui_lista_nome.setCharacterSize(20);
	ui_lista_nome.setPosition(WIDTH-190,50);


	sf::Text ui_lives_label;
	ui_lives_label.setFont(font);
	ui_lives_label.setString("VITE : ");
	ui_lives_label.setCharacterSize(20);
	ui_lives_label.setFillColor(sf::Color(0xbc,0x13,0xfe));
	ui_lives_label.setPosition(WIDTH-280,10);


	sf::Text ui_dizionario;
	ui_dizionario.setFont(font);
	ui_dizionario.setCharacterSize(20);
	ui_dizionario.setFillColor(sf::Color::White);
	ui_dizionario.setPosition(160,50);

	sf::Text ui_destroy_w_counter;
	ui_destroy_w_counter.setFont(font);
	ui_destroy_w_counter.setCharacterSize(20);
	ui_destroy_w_counter.setFillColor(sf::Color::White);

	sf::Text ui_lives;
	ui_lives.setFont(font);
	ui_lives.setCharacterSize(20);
	ui_lives.setFillColor(sf::Color::White);
	ui_lives.setPosition(WIDTH-205	,10);

	sf::Text ui_errors;
	ui_errors.setFont(font);
	ui_errors.setCharacterSize(20);
	ui_errors.setFillColor(sf::Color::White);

	sf::Text u_input;
	u_input.setFont(font);
	u_input.setFillColor(sf::Color::White);
	u_input.setPosition(100,450);
	u_input.setCharacterSize(20);

	sf::Text enterLabel;
	enterLabel.setString("ENTER : ");
	enterLabel.setFont(font);
	enterLabel.setFillColor(sf::Color::Red);
	enterLabel.setPosition(10,450);
	enterLabel.setCharacterSize(20);

	sf::Text end_msg;
	end_msg.setFont(font);
	end_msg.setString("FINISH!");
	end_msg.setCharacterSize(70);
	end_msg.setFillColor(sf::Color::White);
	end_msg.setPosition(WIDTH/2-125,50);

	sf::Text u_speed_label;
	u_speed_label.setFont(font);
	u_speed_label.setString("LA TUA VELOCITA' DI SCRITTURA : ");
	u_speed_label.setCharacterSize(25);
	u_speed_label.setFillColor(sf::Color::Red);
	u_speed_label.setPosition(WIDTH/2-210,350);

	sf::Text press_space;
	press_space.setFont(font);
	press_space.setString("PREMI [ SPACE ] PER RIPROVARE!");
	press_space.setCharacterSize(30);
	press_space.setFillColor(sf::Color::Green);
	press_space.setPosition(WIDTH/2-220,HEIGHT/2+200);

	sf::Text u_speed;
	u_speed.setFont(font);
	u_speed.setCharacterSize(30);
	u_speed.setFillColor(sf::Color::Yellow);
	u_speed.setPosition(WIDTH/2-60,400);

	populateVector(words,font,argv);

	tot_dizionario=words.size();

	Display display(WIDTH,HEIGHT);

	while(window.isOpen())
	{

		u_input.setString(u_input_string);

		if(main_chrono.getElapsedTime().asSeconds() < 10)
			ui_game_time.setString(std::to_string(main_chrono.getElapsedTime().asSeconds()).erase(4));

		else if(main_chrono.getElapsedTime().asSeconds() < 100)
			ui_game_time.setString(std::to_string(main_chrono.getElapsedTime().asSeconds()).erase(5));

		else ui_game_time.setString(std::to_string(main_chrono.getElapsedTime().asSeconds()).erase(6));


		ui_dizionario.setString(std::to_string(tot_dizionario));
		ui_destroy_w_counter.setString(std::to_string(destroyed_w_counter));
		ui_lives.setString(std::to_string(lives));
		ui_errors.setString(std::to_string(errors_counter));

		if(difficult_chrono.getElapsedTime().asSeconds() > 60)
		{
			if(move_time > 0.1) move_time-=0.2;
			if(spawn_time > 1.0) spawn_time-=0.6;
			difficult_chrono.restart();
		}

		if(!is_game_start)
		{
			game_chrono.restart();
			is_game_start=isGameStart(words);
		}

		if(lives == 0 || words.size() == 0) finish=true;

		sf::Event event;

		while(window.pollEvent(event))
		{
			if(event.type == sf::Event::Closed) window.close();
			if(sf::Keyboard::isKeyPressed(sf::Keyboard::Space) && finish) restart(argv);

			else if(event.type == sf::Event::TextEntered)
			{
				if (event.text.unicode < 128 && event.text.unicode != 8 && event.text.unicode != 13 && u_input_string.size() < 30)
				{
					u_input_string+=std::tolower((char) event.text.unicode);
					u_input_string=(std::string) u_input_string;
				}

			}

			else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Enter))
			{
				deleteWord(words,u_input_string);
				u_input_string.clear();
			}


			else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Backspace))
				if(u_input_string.size() != 0) u_input_string.pop_back();



		}

		window.clear();

		if(!finish)
		{
			if(move_chrono.getElapsedTime().asSeconds() > move_time)
			{moveWords(words); move_chrono.restart();}

			if(spawn_chrono.getElapsedTime().asSeconds() > spawn_time)
			{selectWords(words);spawn_chrono.restart();}

			game_time_label.setPosition(10,10);
			ui_game_time.setPosition(80,10);

			window.draw(game_time_label);
			window.draw(ui_game_time);

			window.draw(ui_dizionario_label);
			window.draw(ui_dizionario);
			window.draw(ui_lives);

			ui_destroy_w_label.setPosition(350,50);
			ui_destroy_w_counter.setPosition(480,50);

			errors_label.setPosition(350,10);
			ui_errors.setPosition(480,10);

			window.draw(errors_label);
			window.draw(ui_errors);

			window.draw(ui_destroy_w_counter);
			window.draw(ui_destroy_w_label);

			window.draw(enterLabel);
			window.draw(u_input);

			window.draw(ui_lista_nome_label);
			window.draw(ui_lista_nome);

			window.draw(ui_lives_label);
			display.draw(window);

			checkWordsPosition(words);
			drawWords(words,window);
		}

		if(finish)
		{
			if(game_time_final == 0.0) game_time_final=main_chrono.getElapsedTime().asSeconds();

			std::string final_time_string=std::to_string(game_time_final);

			if(game_time_final < 10) final_time_string.erase(4);
			else if(game_time_final < 100) final_time_string.erase(5);
			else final_time_string.erase(6);

			sf::Text ui_final_time;
			ui_final_time.setFont(font);
			ui_final_time.setString(final_time_string+"s");
			ui_final_time.setCharacterSize(20);
			ui_final_time.setPosition(WIDTH/2+100,150);

			your_speed=(type_time/destroyed_w_counter-0.04444);
			if(destroyed_w_counter > 0) u_speed.setString(std::to_string(your_speed).erase(4)+" p/s");
			else u_speed.setString("N/D");

			game_time_label.setPosition(WIDTH/2-70,150);


			ui_destroy_w_label.setPosition(WIDTH/2-70,200);
			ui_destroy_w_counter.setPosition(WIDTH/2+100,200);

			errors_label.setPosition(WIDTH/2-70,250);
			ui_errors.setPosition(WIDTH/2+100,250);


			window.draw(end_msg);
			window.draw(game_time_label);
			window.draw(ui_final_time);
			window.draw(ui_destroy_w_label);
			window.draw(ui_destroy_w_counter);
			window.draw(errors_label);
			window.draw(ui_errors);
			window.draw(u_speed_label);
			window.draw(u_speed);

			if(game_chrono.getElapsedTime().asSeconds() > 1) window.draw(press_space);
			if(game_chrono.getElapsedTime().asSeconds() > 2) game_chrono.restart();

		}

		window.display();

	}

	return 0;
}

